<?php

/**
 * @file
 * ContextIOMessageParser related classes.
 */

/**
 * Parse Context.IO specific email message data.
 */
class ContextIOMessageParser extends FeedsParser {

  /**
   * Implements FeedsParser::parse().
   */
  public function parse(FeedsSource $source, FeedsFetcherResult $fetcher_result) {
    $fetcher = $source->importer()->fetcher;
    // Check fetcher's type.
    if ((get_class($fetcher) == 'ContextIOMessageFetcher') && !empty($fetcher_result->getMessage())) {
      $message = $fetcher_result->getMessage();
      // Result defaults.
      $result = array(
        'id' => $message['message_id'],
        'date' => $message['date'],
        'subject' => $message['subject'],
        'from_email' => empty($message['addresses']['from']['email']) ? '' : $message['addresses']['from']['email'],
        'from_name' => empty($message['addresses']['from']['name']) ? '' : $message['addresses']['from']['name'],
        'to_emails' => '',
        'flags' => '',
        'body_plain' => '',
        'body_html' => '',
      );

      // Make a list from "To" email addresses.
      if (!empty($message['addresses']['to'])) {
        $to_count = count($message['addresses']['to']) - 1;
        foreach ($message['addresses']['to'] as $index => $to) {
          $result['to_emails'] .= $to['email'];
          if ($index < $to_count) {
            $result['to_emails'] .= ', ';
          }
        }
      }

      // Make a list from flags.
      if (!empty($message['flags'])) {
        $result['flags'] = implode(', ', $message['flags']);
      }

      // Check message bodies. For now plain text and HTML type supported.
      if (!empty($message['body'])) {
        foreach ($message['body'] as $body) {
          switch ($body['type']) {
            case 'text/plain':
              $result['body_plain'] = $body['content'];
              break;

            case 'text/html':
              $result['body_html'] = $body['content'];
              break;
          }
        }
      }

      // Add derivative sources and perform regular expressions.
      foreach ($this->config['derivative_sources'] as $derivative_source) {
        $pattern = $derivative_source['regex']['data']['pattern'];
        $replacement = $derivative_source['regex']['data']['replacement'];
        $derivative_machine_name = $derivative_source['derivative']['data']['machine_name'];
        $derivative_source = $derivative_source['source']['data'];
        $result[$derivative_machine_name] = preg_replace('/' . $pattern . '/', $replacement, $result[$derivative_source]);
      }

      return new FeedsParserResult(array($result));
    }
    return new FeedsParserResult(array());
  }

  /**
   * Return mapping sources.
   *
   * @param bool $with_derivatives
   *   Boolean indicating the result should include derivative sources or not.
   *
   * @return array
   *   An array of available mapping sources.
   */
  public function getMappingSources($with_derivatives = TRUE) {
    $sources = array();
    $fetcher = feeds_importer($this->id)->fetcher;
    // Check fetcher' type.
    if (get_class($fetcher) == 'ContextIOMessageFetcher') {
      // Default sources.
      $sources += array(
        'id' => array(
          'name' => t('Id'),
          'description' => t('The unique and persistent id assigned by Context.IO to the message.'),
        ),
        'date' => array(
          'name' => t('Date'),
          'description' => t('The Unix timestamp the message sent from the origin.'),
        ),
        'subject' => array(
          'name' => t('Subject'),
          'description' => t('The subject of the email message.'),
        ),
        'from_email' => array(
          'name' => t('From - Email'),
          'description' => t('The email address the message received from.'),
        ),
        'from_name' => array(
          'name' => t('From - Name'),
          'description' => t('The account name the message received from (not always available).'),
        ),
        'to_emails' => array(
          'name' => t('To emails'),
          'description' => t('A comma separated email address list the message sent to (not always available).'),
        ),
      );
      // Optionally add bodies related sources.
      if (!empty($fetcher->config['additional_options']['include_body'])) {
        $sources['body_plain'] = array(
          'name' => t('Body - Plain text'),
          'description' => t('The body of the email message in plain text format.'),
        );
        $sources['body_html'] = array(
          'name' => t('Body - HTML'),
          'description' => t('The body of the email message in HTML format.'),
        );
      }
      // Optionally add a flags related source.
      if (!empty($fetcher->config['additional_options']['include_flags'])) {
        $sources['flags'] = array(
          'name' => t('Flags'),
          'description' => t('A comma separated IMAP flag list the message marked with.'),
        );
      }
      // Optionally add derivative sources.
      if ($with_derivatives) {
        foreach ($this->config['derivative_sources'] as $derivative_source) {
          $sources[$derivative_source['derivative']['data']['machine_name']] = array(
            'name' => check_plain($derivative_source['derivative']['data']['name']),
            'description' => check_plain($derivative_source['derivative']['data']['description']),
          );
        }
      }
    }
    return $sources;
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    return array(
      'derivative_sources' => array(),
    );
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = array('#tree' => TRUE);

    $form['derivative_sources_description'] = array(
      '#prefix' => '<p>',
      '#markup' => t(
          'Define derivative sources which will provide additional custom fields
           at processor plugins related mapping settings.'
      ),
      '#suffix' => '</p><br/>',
    );
    $form['derivative_sources'] = array(
      '#theme' => 'contextio_feeds_plugins_message_parser_derivative_sources_table',
      '#header' => array(
        t('Source'),
        t('Derivative'),
        t('Regex'),
        t('Remove'),
      ),
      '#attributes' => array(
        'class' => array('derivative-sources'),
      ),
      '#caption' => NULL,
      '#colgroups' => array(),
      '#sticky' => FALSE,
      '#empty' => NULL,
      '#attached' => array(
        'css' => array(
          drupal_get_path('module', 'contextio_feeds_plugins') . '/css/message_parser_config_form.css',
        ),
      ),
    );

    // Get mapping sources without derivatives.
    $mapping_sources_without_derivatives = $this->getMappingSources(FALSE);

    // Get used source machine names.
    foreach ($this->config['derivative_sources'] as $source) {
      $source_name = $source['source']['data'];
      if (isset($mapping_sources_without_derivatives[$source['source']['data']])) {
        $source_name = check_plain($mapping_sources_without_derivatives[$source['source']['data']]['name']) . ' (' . $source_name . ')';
      }
      // List previously saved derivative sources.
      $form['derivative_sources'][] = array(
        'source' => array(
          '#markup' => $source_name,
        ),
        'derivative' => array(
          'name' => array(
            '#markup' => check_plain($source['derivative']['data']['name']) . ' (' . $source['derivative']['data']['machine_name'] . ')',
          ),
          'description' => array(
            '#prefix' => '<div class="description">',
            '#markup' => check_plain($source['derivative']['data']['description']),
            '#suffix' => '</div>',
          ),
        ),
        'regex' => array(
          'pattern' => array(
            '#prefix' => '<p>',
            '#markup' => t('Pattern: /@pattern/', array(
                '@pattern' => $source['regex']['data']['pattern'],
            )),
            '#suffix' => '</p>',
          ),
          'replacement' => array(
            '#prefix' => '<p>',
            '#markup' => t('Replacement: @replacement', array(
              '@replacement' => $source['regex']['data']['replacement'],
            )),
            '#suffix' => '</p>',
          ),
        ),
        'remove' => array(
          'data' => array(
            '#type' => 'checkbox',
            '#return_value' => $source['derivative']['data']['machine_name'],
          ),
          'class' => array(
            '#value' => 'remove',
          ),
        ),
      );
    }

    // Get available source options.
    $derivative_sources_source_options = array();
    foreach ($mapping_sources_without_derivatives as $key => $source) {
      $derivative_sources_source_options[$key] = $source['name'] . ' (' . $key . ')';
    }

    $form['derivative_sources'][] = array(
      'source' => array(
        'data' => array(
          '#type' => 'select',
          '#title' => t('Source'),
          '#options' => $derivative_sources_source_options,
          '#empty_option' => t('- Select -'),
          '#description' => t('The source of the derivative.'),
        ),
        'class' => array(
          '#value' => 'source',
        ),
      ),
      'derivative' => array(
        'data' => array(
          'name' => array(
            '#type' => 'textfield',
            '#title' => t('Name'),
            '#description' => t('The name of the derivative.'),
          ),
          'machine_name' => array(
            '#type' => 'machine_name',
            '#machine_name' => array(
              'source' => array(
                'derivative_sources',
                count(element_children($form['derivative_sources'])),
                'derivative',
                'data',
                'name',
              ),
              'exists' => 'contextio_feeds_plugins_derivative_source_exists',
            ),
            '#description' => t('The machine-readable name the derivative.'),
            '#used_machine_names' => array_keys($this->getMappingSources()),
            '#required' => FALSE,
          ),
          'description' => array(
            '#type' => 'textfield',
            '#title' => t('Description'),
            '#description' => t('Description text for the derivative.'),
            '#maxlength' => 255,
          ),
        ),
        'class' => array(
          '#value' => 'derivative',
        ),
      ),
      'regex' => array(
        'data' => array(
          'pattern' => array(
            '#type' => 'textfield',
            '#title' => t('Pattern'),
            '#default_value' => NULL,
            '#description' => t(
              "The regex pattern to search for. Do not include leading or closing pattern wrapper characters
              like slash (/)! Parenthesis can be used for references in the replacement field."
            ),
          ),
          'replacement' => array(
            '#type' => 'textfield',
            '#title' => t('Replacement'),
            '#default_value' => NULL,
            '#description' => t(
              "The string to replace. May contain references of the form \$n or \${n}. Every
              such reference will be replaced by the text captured by the n'th parenthesized pattern in the
              pattern field. n can be from 0 to 99, and \$0 or \${0} refers to the text matched by the whole
              pattern."
            ),
          ),
        ),
        'class' => array(
          '#value' => 'regex',
        ),
      ),
      'remove' => array(
        'data' => array(
          '#markdown' => '&nbsp;',
        ),
      ),
    );

    return $form;
  }

  /**
   * Override parent::configFormValidate().
   */
  public function configFormValidate(&$values) {
    $new_derivative_source = FALSE;
    $new_derivative_source_errors = array();
    $new_derivative_source_index = count($values['derivative_sources']) - 1;
    $new_derivative_source_values = $values['derivative_sources'][$new_derivative_source_index];

    // Check new derivative source related values.
    if (!empty($new_derivative_source_values['source']['data'])) {
      $new_derivative_source = TRUE;
    }
    else {
      $new_derivative_source_errors['source][data'] = t('Source cannot be empty.');
    }

    if (!empty($new_derivative_source_values['derivative']['data']['name'])) {
      $new_derivative_source = TRUE;
    }
    else {
      $new_derivative_source_errors['derivative][data][name'] = t('Name cannot be empty.');
    }

    if (!empty($new_derivative_source_values['derivative']['data']['machine_name'])) {
      $new_derivative_source = TRUE;
    }
    else {
      $new_derivative_source_errors['derivative][data][machine_name'] = t('Machine name cannot be empty.');
    }

    if (!empty($new_derivative_source_values['regex']['data']['pattern'])) {
      $new_derivative_source = TRUE;
    }
    else {
      $new_derivative_source_errors['regex][data][pattern'] = t('Pattern cannot be empty.');
    }

    if (!empty($new_derivative_source_values['regex']['data']['replacement'])) {
      $new_derivative_source = TRUE;
    }
    else {
      $new_derivative_source_errors['regex][data][replacement'] = t('Replacement cannot be empty.');
    }

    // Check new derivative source related form errors.
    if ($new_derivative_source) {
      if (!empty($new_derivative_source_errors)) {
        foreach ($new_derivative_source_errors as $key => $message) {
          form_set_error('derivative_sources][' . $new_derivative_source_index . '][' . $key, $message);
        }
      }
      else {
        // Save values for later usage.
        $values['#new_derivative_source_values'] = $new_derivative_source_values;
      }
    }
  }

  /**
   * Override parent::configFormSubmit().
   */
  public function configFormSubmit(&$values) {
    $config_modified = FALSE;

    // Check removable derivative sources.
    $remove_derivative_sources = array();
    foreach ($values['derivative_sources'] as $source) {
      if (!empty($source['remove']['data'])) {
        $remove_derivative_sources[] = $source['remove']['data'];
      }
    }
    // Remove derivative sources.
    if (!empty($remove_derivative_sources)) {
      foreach ($this->config['derivative_sources'] as $index => $saved_source) {
        if (in_array($saved_source['derivative']['data']['machine_name'], $remove_derivative_sources)) {
          unset($this->config['derivative_sources'][$index]);
          $config_modified = TRUE;
        }
      }
    }

    // Add new derivative source.
    if (!empty($values['#new_derivative_source_values'])) {
      // Use own submit functionality instead of the parent's one.
      $this->config['derivative_sources'][] = $values['#new_derivative_source_values'];
      $config_modified = TRUE;
    }

    // Save config changes.
    if ($config_modified) {
      $this->setConfig($this->config);
      $this->save();
      drupal_set_message(t('Your changes have been saved.'));
      feeds_cache_clear(FALSE);
    }
  }

}
